<?php
namespace NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget;

use \Magento\Backend\Model\View\Result\Redirect;
use \Magento\Framework\Controller\ResultInterface;
use \NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget;
use \NicolasBejean\CategoryWidget\Model\CategoryWidget as CategoryWidgetModel;
use \Exception;

/**
 * Class Delete
 *
 * @category PHP
 * @package  NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget
 * @author   Nicolas Béjean <nicolas@bejean.eu>
 * @license  https://github.com/nicolasbejean/category-widget/blob/master/licence.txt BSD Licence
 * @link     https://www.bejean.eu
 */
class Delete extends CategoryWidget
{
    /**
     * Delete action
     *
     * @return ResultInterface
     */
    public function execute()
    {
        /** @var Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        // check if we know what should be deleted
        $id = $this->getRequest()->getParam('entity_id');
        if ($id) {
            try {
                /** @var CategoryWidgetModel $model */
                $model = $this->_objectManager->create(CategoryWidgetModel::class);
                $model->load($id);
                $model->delete();
                // display success message
                $this->messageManager->addSuccessMessage(__('You deleted the category widget.'));
                // go to grid
                return $resultRedirect->setPath('*/*/');
            } catch (Exception $e) {
                // display error message
                $this->messageManager->addErrorMessage($e->getMessage());
                // go back to edit form
                return $resultRedirect->setPath('*/*/edit', ['entity_id' => $id]);
            }
        }
        // display error message
        $this->messageManager->addErrorMessage(__('We can\'t find an category widget to delete.'));
        // go to grid
        return $resultRedirect->setPath('*/*/');
    }
}
