<?php
namespace NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget;

use \Magento\Backend\Model\View\Result\Redirect;
use \Magento\Framework\Controller\ResultInterface;
use \NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget as CategoryWidgetController;
use \Magento\Backend\App\Action\Context;
use \NicolasBejean\CategoryWidget\Api\CategoryWidgetRepositoryInterface;
use \NicolasBejean\CategoryWidget\Model\CategoryWidget;
use \NicolasBejean\CategoryWidget\Model\CategoryWidgetFactory;
use \Magento\Framework\App\Request\DataPersistorInterface;
use \Magento\Framework\Registry;
use \Magento\Framework\App\ObjectManager;
use \Exception;

/**
 * Class Save
 *
 * @category PHP
 * @package  NicolasBejean\CategoryWidget\Controller\Adminhtml\CategoryWidget
 * @author   Nicolas Béjean <nicolas@bejean.eu>
 * @license  https://github.com/nicolasbejean/category-widget/blob/master/licence.txt BSD Licence
 * @link     https://www.bejean.eu
 */
class Save extends CategoryWidgetController
{
    /**
     * @var DataPersistorInterface
     */
    protected $dataPersistor;

    /**
     * @var CategoryWidgetFactory
     */
    private $categoryWidgetFactory;

    /**
     * @var CategoryWidgetRepositoryInterface
     */
    private $categoryWidgetRepository;

    /**
     * @param Context $context
     * @param Registry $coreRegistry
     * @param DataPersistorInterface $dataPersistor
     * @param CategoryWidgetFactory|null $categoryWidgetFactory
     * @param CategoryWidgetRepositoryInterface|null $categoryWidgetRepository
     */
    public function __construct(
        Context $context,
        Registry $coreRegistry,
        DataPersistorInterface $dataPersistor,
        CategoryWidgetFactory $categoryWidgetFactory = null,
        CategoryWidgetRepositoryInterface $categoryWidgetRepository = null
    ) {
        $this->dataPersistor = $dataPersistor;
        $this->categoryWidgetFactory = $categoryWidgetFactory
            ?: ObjectManager::getInstance()->get(CategoryWidgetFactory::class);
        $this->categoryWidgetRepository = $categoryWidgetRepository
            ?: ObjectManager::getInstance()->get(CategoryWidgetRepositoryInterface::class);
        parent::__construct($context, $coreRegistry);
    }

    /**
     * Save action
     *
     * @SuppressWarnings(PHPMD.CyclomaticComplexity)
     * @return ResultInterface
     */
    public function execute()
    {
        /** @var Redirect $resultRedirect */
        $resultRedirect = $this->resultRedirectFactory->create();
        $data = $this->getRequest()->getPostValue();
        if ($data) {

            if (isset($data['is_active']) && $data['is_active'] === 'true') {
                $data['is_active'] = CategoryWidget::STATUS_ENABLED;
            }
            if (empty($data['entity_id'])) {
                $data['entity_id'] = null;
            }

            /** @var CategoryWidget $model */
            $model = $this->categoryWidgetFactory->create();

            $id = $this->getRequest()->getParam('entity_id');
            if ($id) {
                try {
                    $model = $this->categoryWidgetRepository->getById($id);
                } catch (Exception $e) {
                    $this->messageManager->addErrorMessage(__('This category widget no longer exists.'));
                    return $resultRedirect->setPath('*/*/');
                }
            }

            $model->setData($data);

            try {
                $this->categoryWidgetRepository->save($model);
                $this->messageManager->addSuccessMessage(__('You saved the category widget.'));
                $this->dataPersistor->clear('nicolasbejean_categorywidget_item');
                return $this->processCategoryWidgetReturn($model, $data, $resultRedirect);
            } catch (Exception $e) {
                $this->messageManager->addExceptionMessage($e, __('Something went wrong while saving the category widget.'));
            }

            $this->dataPersistor->set('nicolasbejean_categorywidget_item', $data);
            return $resultRedirect->setPath('*/*/edit', ['entity_id' => $id]);
        }
        return $resultRedirect->setPath('*/*/');
    }

    /**
     * Process and set the category widget return
     *
     * @param CategoryWidget $model
     * @param array $data
     * @param ResultInterface $resultRedirect
     * @return ResultInterface
     */
    private function processCategoryWidgetReturn($model, $data, $resultRedirect)
    {
        $redirect = $data['back'] ?? 'close';

        if ($redirect ==='continue') {
            $resultRedirect->setPath('*/*/edit', ['entity_id' => $model->getId()]);
        } elseif ($redirect === 'close') {
            $resultRedirect->setPath('*/*/');
        } elseif ($redirect === 'new') {
            $resultRedirect->setPath('*/*/new');
        } elseif ($redirect === 'duplicate') {
            $duplicateModel = $this->categoryWidgetFactory->create(['data' => $data]);
            $duplicateModel->setId(null);
            $duplicateModel->setIdentifier($data['identifier'] . '-' . uniqid());
            $duplicateModel->setName($data['name']);
            $duplicateModel->setContent($data['content']);
            $duplicateModel->setIsActive(CategoryWidget::STATUS_DISABLED);
            $this->categoryWidgetRepository->save($duplicateModel);
            $id = $duplicateModel->getId();
            $this->messageManager->addSuccessMessage(__('You duplicated the category widget.'));
            $this->dataPersistor->set('nicolasbejean_categorywidget_item', $data);
            $resultRedirect->setPath('*/*/edit', ['entity_id' => $id]);
        }
        return $resultRedirect;
    }
}
